;**************************************************************************
;*
;* FILE  loader.asm
;* Copyright (c) 2009 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: loader.asm,v 1.11 2009-08-23 10:27:21 tlr Exp $
;*
;* DESCRIPTION
;*   VIC20 tinyrs loader
;*   Configurations are:
;*     Unexp: $1001-$1dff
;*     +3Kb:  $0401-$1dff
;*     +8Kb:  $1201-$1fff
;*
;******
	processor 6502

RELOC_BYTE	equ	$40
msb_zp	equ	$fe


	seg	code
	org	$1201
begin:
;**************************************************************************
;*
;* Basic line!
;*
;******
start_of_line:
	dc.w	end_line
	dc.w	0
	dc.b	$9e,"("		; SYS(
	dc.b	$c2,"(44)"	; PEEK(44)
	dc.b	$ac,"256"  	; *256
	dc.b	$aa,"30)"  	; +30)
	dc.b	"/T.L.R/",0
end_line:
	dc.w	0
	; fall thru

;**************************************************************************
;*
;* NAME  startofcode
;*
;******
startofcode:
	sei
INSTALL_OFFS_LSB equ	install_st - begin
INSTALL_END_LSB	 equ	install_end - begin
	ldy	#INSTALL_OFFS_LSB
soc_lp1:
	lda	($2b),y		; basic start ($1201, $1001 or $0401)
	sta	install-INSTALL_OFFS_LSB,y
	iny
	cpy	#INSTALL_END_LSB
	bne	soc_lp1
; C=1
	jmp	install_entry


install_st:
	rorg	$0110
;**************************************************************************
;*
;* NAME  install
;*
;******
install:

install_entry:
; Set source address from basic pointer
	lda	$2c
	sta	ie_sm1+2

;******
;* Check for PAL or NTSC
;*
; find raster line 2
	lda	#2/2
cp_lp1:
	cmp	$9004
	bne	cp_lp1
; now we know that we are past raster line 0, see if we find raster line 268
; before we find raster line 0
cp_lp2:	
	lda	$9004
	beq	cp_ex1		; Line 0 reached, we must be NTSC.
	cmp	#268/2		; This line does not exist on NTSC.
	bne	cp_lp2
; Line 268 reached, we must be PAL.

; setup loader for PAL.
	lda	#<payload_pal
	sta	ie_sm1+1
	inc	ie_sm1+2
	
cp_ex1:	

;******
;* lower memtop by at least 256 bytes (to $xx00) and return pointer in X/Y.
;*
	sec
	jsr	$ff99
	dey
	ldx	#0
	clc
	jsr	$ff99
; X=0
	sty	msb_zp
	sty	ie_sm2+2

;******
;* copy payload to target address
;*
;	ldx	#0		
ie_lp1:
ie_sm1:
	lda.w	<payload_ntsc,x
	cmp	#RELOC_BYTE
	bne	ie_skp1
	tya
ie_skp1:
ie_sm2:
	sta.w	$0000,x
	inx
	bne	ie_lp1

;******
;* print messages
;*
ie_lp2:
	jsr	get_msg
	jsr	$ffd2
	bne	ie_lp2
	jsr	get_msg
	sta	ie_sm3+1
ie_sm3:
	jmp	ie_sm3

ie_psys0:
	ldx	#0
	dc.b	$2c
ie_psys3:
	ldx	#3
	lda	msb_zp
	jsr	$ddcd
	jmp	ie_lp2

ie_exit:
;******
;* perform new and exit to basic
;*
	jsr	$c644	;NEW
	jmp	$c474

greet_msg:
	dc.b	13,156
	dc.b	"  ",18,"TINYRS      TLR",13
	dc.b	"  ",18,"38K4   PAL/NTSC",13
	dc.b	28,"R",31,":SYS",0,<ie_psys0," ",28,"W",31,":SYS",0,<ie_psys3,13
	dc.b	0,<ie_exit

get_msg:
	inc	gm_sm1+1
gm_sm1:
	lda	greet_msg-1
	rts

	rend
install_end:
INSTALL_LEN	equ	install_end-install_st
	echo "install",install,install+INSTALL_LEN	


;**************************************************************************
;*
;* NAME  payload
;*
;******
payload:
payload_ntsc:
	incbin	"tinyrs_core_ntsc.prg",2
payload_pal:
	incbin	"tinyrs_core_pal.prg",2
PAYLOAD_LEN	equ	.-payload

	echo	.
; eof
