;**************************************************************************
;*
;* FILE  tinyrs.asm
;* Copyright (c) 2009 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: tinyrs.asm,v 1.28 2009-08-25 19:42:51 tlr Exp $
;*
;* DESCRIPTION
;*   Tiny fastrs clone for the VIC-20.
;*   Implements the over5 simpleread/simplewrite protocol. 	
;*
;******
	processor 6502

	ifnconst ROMABLE
ADDR	equ	$4000
	else
ADDR	equ	$9f00
	endif

	seg.u	zp
	org	$f7
shift_zp:
	ds.b	1
cnt_zp:
	ds.b	1
ptr_zp:
end_zp	equ	ptr_zp+2
	ds.w	2
chk_zp:
	ds.b	1
dummy_zp:
	ds.b	1

	seg	code
	org	ADDR
payload:
;**************************************************************************
;*
;* NAME  jump table
;*
;******
	jmp	receive_file
; 	jmp	send_file	
	; fall through

;**************************************************************************
;*
;* NAME  send_file
;*
;* DESCRIPTION
;*
;******
send_file:
	lda	#"W"
	jsr	init
; X=0

; chk_zp =0
	lda	#$e7
	jsr	send_byte

; X=0
; 	ldx	#0
sf_lp1:
	lda	$2b,x
	sta	ptr_zp,x
	stx	cnt_zp
	jsr	send_byte
	ldx	cnt_zp
	inx
	cpx	#4
	bne	sf_lp1

	jsr	send_chk		; checksum

; Here chk_zp == 0.

sf_lp2:
	ldy	#0
	lda	(ptr_zp),y
	jsr	send_byte
	inc	ptr_zp
	bne	sf_skp1
	inc	ptr_zp+1
sf_skp1:
	lda	ptr_zp
	cmp	end_zp
	lda	ptr_zp+1
	sbc	end_zp+1
	bcc	sf_lp2

	jsr	send_chk		; checksum

exit_ok:
	ldy	#$6a
	jmp	$f1e6		; Print OK and do rts.
;	rts

;**************************************************************************
;*
;* NAME  receive_file
;*
;* DESCRIPTION
;*
;******
receive_file:
	lda	#"R"
	jsr	init
; X=0

; shift_zp = 0, chk_zp =0
	jsr	recv_set_byte	; (increments X)
	cmp	#$e7		; header id
	bne	rf_fl1
; X=1

; 	ldx	#1
rf_lp1:
	jsr	receive		; ptr_zp, end_zp (increments X)
	sta	ptr_zp-2,x
	cpx	#4+1
	bne	rf_lp1

	jsr	receive		; checksum
	cmp	chk_zp
	bne	rf_fl1

; Here chk_zp == shift_zp.   When we enter the next receive 
; recv_block chk_zp will be made 0 automatically.

	ldx	ptr_zp
	ldy	ptr_zp+1
	lda	#$8d		; STA <abs>
	jsr	recv_set
	stx	$2d
	sty	$2e

	jsr	recv_set_byte
	cmp	chk_zp
	beq	exit_ok

rf_fl1:
	jmp	$e19c		; Print "?LOAD  ERROR" and exit to basic
;	rts

;**************************************************************************
;*
;* NAME  init
;*
;* DESCRIPTION
;*   Initialize user port pins for RS-232 communication.
;*
;******
init:
	jsr	$ffd2
	sei

	lda	#%00000110
	sta	$9112	;RS232 Data dir (bit 0=RxD)
	lda	#%11111110
	sta	$911c	;set Txd HI

	ifconst	ROMABLE
	ldx	#RAMCODE_LEN
in_lp1:
	lda	ramcode_st-1,x
	sta	ramcode-1,x
	dex
	bne	in_lp1
	stx	shift_zp
	stx	chk_zp

	else
	ldx	#0
	stx	shift_zp
	stx	chk_zp

	endif
	rts

	
	ifconst	PAL
;**************************************************************************
;*
;* NAME  PAL timing
;*
;* DESCRIPTION
;*
;* VIC-20 PAL (1108405 hz) 38400 8N2
;* <cycles per bit>  1108405 / 38400 = 28.864
;*
;*          0     29    58    87   115   144   173   202   231   260
;* (cycles)
;*             29    29    29    28    29    29    29    29    29
;*        _____ _____ _____ _____ _____ _____ _____ _____ _____
;*       |     |     |     |     |     |     |     |     |     |
;*       |start|  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |stop  stop
;* ______|_____|_____|_____|_____|_____|_____|_____|_____|_____|_____|_____|_
;*
;******
	mac	PAL_NOP
	nop
	endm
RECV_TIME	equ	%11111011
SEND_TIME	equ	%11110111
	endif
	
	ifconst	NTSC
;**************************************************************************
;*
;* NAME  NTSC timing
;*
;* DESCRIPTION
;*
;* VIC-20 NTSC (1022727 hz) 38400 8N2
;* <cycles per bit>  1022727 / 38400 = 26.634
;*
;*          0     27    53    80   107   133   160   186   213   240
;* (cycles)
;*             27    26    27    27    26    27    26    27    27
;*        _____ _____ _____ _____ _____ _____ _____ _____ _____
;*       |     |     |     |     |     |     |     |     |     |
;*       |start|  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |stop  stop
;* ______|_____|_____|_____|_____|_____|_____|_____|_____|_____|_____|_____|_
;*
;******
	mac	PAL_NOP
	endm
RECV_TIME	equ	%11010110
SEND_TIME	equ	%10101101
	endif

	ifconst	ROMABLE
ramcode_st:
	rorg	$0110
ramcode:
	endif
;**************************************************************************
;*
;* NAME  receive
;*
;* DESCRIPTION
;*
;******
recv_set_byte:
	lda	#$60		; RTS
recv_set:
	sta	rc_sm1+0	; 4
receive:
rc_lp1:
	lda	#%00000001	; 2
rc_lp2:
	bit	$9110		; 4
	bne	rc_lp2		; 2(3)

;---------------- 29+9=38 (PAL) / 27+9=36 (NTSC)
	lda	shift_zp	; 3
	eor	chk_zp		; 3
	sta	chk_zp		; 3

	stx	rc_sm1+1	; 4
	sty	rc_sm1+2	; 4
	inx			; 2
	bne	rc_skp1		; 2(3)
	iny			; 2	=23
rc_skp1:
	lda	#RECV_TIME	; 2
	sta	shift_zp	; 3
	lda	#7		; 2
	sta	cnt_zp		; 3
	bit	$ea		; 3
	PAL_NOP			; 2	=38
;----------------
rc_lp3:
	lda	$9110		; 4
	lsr			; 2
	ror	shift_zp	; 5
	bcs	.+2		; 2(3)
	PAL_NOP			; 2
	inc	dummy_zp	; 5
	dec	cnt_zp		; 5
	bne	rc_lp3		; 3(2)	=28/29
;----------------
	lda	$9110		; 4
	lsr			; 2
	ror	shift_zp	; 5
	lda	shift_zp	; 3
rc_sm1:	sta.w	$0000		; 4
	cpx	end_zp		; 3
	bne	rc_lp1		; 2(3)
	cpy	end_zp+1	; 3
	bne	rc_lp1		; 3(2)	=29
;----------------
	rts

	ifconst	ROMABLE
	rend
RAMCODE_LEN	equ	.-ramcode_st
	endif
do_rts	equ	.-1

;**************************************************************************
;*
;* NAME  send_byte
;*
;* DESCRIPTION
;*   returns with X=0
;*
;******
send_chk:
	lda	chk_zp		; 3
send_byte:
	sta	shift_zp	; 3
	eor	chk_zp		; 3
	sta	chk_zp		; 3
	ldx	#1+8+2		; 2
	lda     #SEND_TIME	; 2
sn_lp1:
	nop			; 2
	ldy     #%11011110	; 2
sn_lp2:
	sty	$911c		; 4
	dex			; 2
	beq	do_rts		; 2(3)
	ror			; 2
	bcs	.+2		; 2(3)
	PAL_NOP			; 2
	sec			; 2
	ror	shift_zp	; 5
	bcc	sn_lp1		; 2(3)
	ldy	#%11111110	; 2
	bne	sn_lp2		; 3	always taken

PAYLOAD_LEN	equ	.-payload
	echo	PAYLOAD_LEN

; eof
